@@ -23,46 +23,19 @@ binaries (needed for exec) and run them capturing outputs.
 
 
 import os
-import subprocess
 from contextlib import contextmanager
 
+from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
+    WindowsError,
+    subprocess,
+)
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general
 
+from .Download import getCachedDownloadedMinGW64
+from .FileOperations import getExternalUsePath
 from .Utils import getArchitecture, getOS, isWin32Windows
 
-
-def callExec(args):
-    """Do exec in a portable way preserving exit code.
-
-    On Windows, unfortunately there is no real exec, so we have to spawn
-    a new process instead.
-    """
-
-    # On Windows os.execl does not work properly
-    if isWin32Windows():
-        args = list(args)
-        del args[1]
-
-        try:
-            process = subprocess.Popen(args=args)
-            process.communicate()
-            # No point in cleaning up, pylint: disable=protected-access
-            try:
-                os._exit(process.returncode)
-            except OverflowError:
-                # Seems negative values go wrong otherwise,
-                # see https://bugs.python.org/issue28474
-                os._exit(process.returncode - 2 ** 32)
-        except KeyboardInterrupt:
-            # There was a more relevant stack trace already, so abort this
-            # right here, pylint: disable=protected-access
-            os._exit(2)
-    else:
-        # The star arguments is the API of execl
-        os.execl(*args)
-
-
 # Cache, so we avoid repeated command lookups.
 _executable_command_cache = {}
 
@@ -78,18 +51,21 @@ def _getExecutablePath(filename, search_path):
     for path_element in path_elements:
         path_element = path_element.strip('"')
 
-        full = os.path.join(path_element, filename)
+        full = os.path.join(os.path.expanduser(path_element), filename)
 
         if os.path.exists(full):
             return full
 
 
-def getExecutablePath(filename):
-    """ Find an execute in PATH environment. """
+def getExecutablePath(filename, extra_dir=None):
+    """Find an execute in PATH environment."""
 
     # Search in PATH environment.
     search_path = os.environ.get("PATH", "")
 
+    if extra_dir is not None:
+        search_path = extra_dir + os.pathsep + search_path
+
     key = (filename, search_path)
     if key not in _executable_command_cache:
         _executable_command_cache[key] = _getExecutablePath(filename, search_path)
@@ -145,6 +121,17 @@ def getPythonExePathWindows(search, arch):
 
 
 class NuitkaCalledProcessError(subprocess.CalledProcessError):
+    def __init__(self, retcode, cmd, output, stderr):
+        # False alarm, pylint: disable=super-init-not-called
+
+        subprocess.CalledProcessError(self, retcode, cmd)
+
+        # Python2 doesn't have this otherwise, but needs it.
+        self.stderr = stderr
+        self.output = output
+        self.cmd = cmd
+        self.returncode = retcode
+
     def __str__(self):
         result = subprocess.CalledProcessError.__str__(self)
 
@@ -205,14 +192,21 @@ def check_call(*popenargs, **kwargs):
         )
 
 
+def callProcess(*popenargs, **kwargs):
+    """Call a process and return result code."""
+    subprocess.call(*popenargs, **kwargs)
+
+
 @contextmanager
 def withEnvironmentPathAdded(env_var_name, *paths):
+    assert os.path.sep not in env_var_name
+
     paths = [path for path in paths if path]
     path = os.pathsep.join(paths)
 
     if path:
         if str is not bytes and type(path) is bytes:
-            path = path.decode("utf-8")
+            path = path.decode("utf8")
 
         if env_var_name in os.environ:
             old_path = os.environ[env_var_name]
@@ -232,7 +226,7 @@ def withEnvironmentPathAdded(env_var_name, *paths):
 
 @contextmanager
 def withEnvironmentVarOverriden(env_var_name, value):
-    """ Change an environment and restore it after context. """
+    """Change an environment and restore it after context."""
 
     if env_var_name in os.environ:
         old_value = os.environ[env_var_name]
@@ -255,7 +249,7 @@ def withEnvironmentVarOverriden(env_var_name, value):
 
 @contextmanager
 def withEnvironmentVarsOverriden(mapping):
-    """ Change multiple environment variables and restore them after context. """
+    """Change multiple environment variables and restore them after context."""
 
     old_values = {}
 
@@ -295,10 +289,24 @@ def wrapCommandForDebuggerForExec(*args):
     """
 
     gdb_path = getExecutablePath("gdb")
-    lldb_path = getExecutablePath("lldb")
 
-    if gdb_path is None and lldb_path is None:
-        general.sysexit("Error, no 'gdb' or 'lldb' binary found in path.")
+    # Windows extra ball, attempt the downloaded one.
+    if isWin32Windows():
+        from nuitka.Options import assumeYesForDownloads
+
+        mingw64_gcc_path = getCachedDownloadedMinGW64(
+            target_arch=getArchitecture(),
+            assume_yes_for_downloads=assumeYesForDownloads(),
+        )
+
+        with withEnvironmentPathAdded("PATH", os.path.dirname(mingw64_gcc_path)):
+            gdb_path = getExecutablePath("gdb")
+
+    if gdb_path is None:
+        lldb_path = getExecutablePath("lldb")
+
+        if lldb_path is None:
+            general.sysexit("Error, no 'gdb' or 'lldb' binary found in path.")
 
     if gdb_path is not None:
         args = (gdb_path, "gdb", "-ex=run", "-ex=where", "-ex=quit", "--args") + args
@@ -399,11 +407,13 @@ def getNullInput():
     try:
         return subprocess.NULLDEV
     except AttributeError:
-        return open(os.devnull, "rb")
+        # File is supposed to stay open, pylint: disable=consider-using-with
+        subprocess.NULLDEV = open(os.devnull, "rb")
+        return subprocess.NULLDEV
 
 
 def executeToolChecked(logger, command, absence_message, stderr_filter=None):
-    """Execute external tool, checking for success and no error outputs, returning result. """
+    """Execute external tool, checking for success and no error outputs, returning result."""
 
     tool = command[0]
 
@@ -428,13 +438,35 @@ def executeToolChecked(logger, command, absence_message, stderr_filter=None):
         stderr = stderr_filter(stderr)
 
     if result != 0:
-        logger.sysexit(
-            "Error, call to %r to change shared library paths failed: %s -> %s."
-            % (tool, command, stderr)
-        )
+        logger.sysexit("Error, call to %r failed: %s -> %s." % (tool, command, stderr))
     elif stderr:
         logger.sysexit(
             "Error, call to %r gave warnings: %s -> %s." % (tool, command, stderr)
         )
 
     return stdout
+
+
+def executeProcess(
+    command, env=None, needs_stdin=False, shell=False, external_cwd=False
+):
+    if not env:
+        env = os.environ
+
+    process = subprocess.Popen(
+        command,
+        stdin=subprocess.PIPE if needs_stdin else getNullInput(),
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        shell=shell,
+        # On Windows, closing file descriptions is now working with capturing outputs.
+        close_fds=not isWin32Windows(),
+        env=env,
+        # For tools that want short paths to work.
+        cwd=getExternalUsePath(os.getcwd()) if external_cwd else None,
+    )
+
+    stdout, stderr = process.communicate()
+    exit_code = process.wait()
+
+    return stdout, stderr, exit_code
